#!/usr/bin/env python
# -*- coding: utf-8 -*-

#    Copyright (C) <2007>  <Sebastian Rittau ><http://www.rittau.org/blog/20070726-01>

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

from Singleton import Singleton
import gnomekeyring as gkey

class Keyring(object):
    __metaclass__ = Singleton

    def __init__(self, name):
        self.__name = name
#        self.__server = server
#        self.__protocol = protocol
        self.__keyring = gkey.get_default_keyring_sync()

    def hasCredentials(self,server,protocol=None):
        try:
            attrs = {"server": server, "protocol": protocol}
            items = gkey.find_items_sync(gkey.ITEM_NETWORK_PASSWORD, attrs)
            return len(items) > 0
        except gkey.DeniedError:
            return False

    def getCredentials(self,server,protocol=None):
        try:
            attrs = None
            if protocol:
                attrs = {"server": server, "protocol": protocol}
            else:
                attrs = {"server": server}
            items = gkey.find_items_sync(gkey.ITEM_NETWORK_PASSWORD, attrs)
            return (items[0].attributes["user"], items[0].secret)
        except gkey.NoMatchError:
            return None

    def setCredentials(self, (user, pw),server,protocol=None):
        attrs = None
        if protocol:
            attrs = {
                "user": user,
                "server": server,
                "protocol": protocol
            }
        else:
            attrs = {
                "user": user,
                "server": server
            }
        gkey.item_create_sync(gkey.get_default_keyring_sync(),
                gkey.ITEM_NETWORK_PASSWORD, self.__name, attrs, pw, True)
#from Keyring import Keyring
#    key = Keyring("PyShare","rapidshare.com","http")
#    print key.get_credentials()
#    key.set_credentials()

