#!/usr/bin/env python
# -*- coding: utf-8 -*-

#    Copyright (C) <2009>  <Sebastian Kacprzak> <naicik |at| gmail |dot| com>

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pygtk
pygtk.require("2.0")
import gtk.glade
from helpers.gtkHelper import createCombobox
from Settings import Settings

__author__ = "Sebastian Kacprzak <naicik at gmail.com>"
__date__ = "$2009-12-12 14:22:55$"

import gettext
translationsDomainName = 'Settings_GTK' #glade and python file use same domain name, because for user it is the same window
translationsDirName = 'translations'
gettext.install(translationsDomainName, translationsDirName)

class Settings_GTK():

    def __init__(self, caller=None):
        self.__set = Settings()
        self.__settings = self.__set.getSettings()

        self.__parent = caller

        gtk.glade.bindtextdomain(translationsDomainName, translationsDirName)

        gladefile = "settings.glade"
        self.__wTree = gtk.glade.XML(gladefile)

        self.__window = self.__wTree.get_widget("Settings")
        self.__optipngBox = self.__wTree.get_widget("hboxOptipng")
        self.__comboboxFileType = self.__wTree.get_widget("comboboxFileType")
        self.__hscaleCompression = self.__wTree.get_widget("hscaleCompression")
        self.__labelScaleMax = self.__wTree.get_widget("labelScaleMax")

        self.__window.connect("destroy", self.__destroy)

        dic = {
            "on_buttonOK_clicked": self.__buttonOKClicked,
#            "on_buttonCancel_clicked": self.__buttonCancelClicked,
            "on_buttonReset_clicked": self.__buttonResetClicked,
            "on_hscaleCompression_value_changed": self.__scaleCompressionValueChanged,
            "on_comboboxFileType_changed": self.__comboboxFileTypeChanged,
            "on_buttonInstallOptipng_clicked": self.__buttonInstallOptipng,
            "on_buttonGetCurrentHeight_clicked": self.__buttonGetCurrentHeightClicked,
            "on_spinbuttonNumberOfConnections_value_changed":self.__spinbuttonNumberOfConnections_value_changed,
            "on_comboboxClipboard_changed":self.__comboboxClipboard_changed,
            "on_checkbuttonOneInstance_toggled":self.__checkbuttonOneInstance_toggled,
            "on_spinbuttonHeight_value_changed":self.__spinbuttonHeight_value_changed,
            "on_spinbuttonThumb_value_changed":self.__spinbuttonThumb_value_changed,
            "on_checkbuttonNotification_toggled":self.__checkbuttonNotification_toggled,
            "on_checkbuttonIcon_toggled":self.__checkbuttonIcon_toggled,
            "on_checkbuttonBlink_toggled":self.__checkbuttonBlink_toggled,
            "on_checkbuttonUseProxy_toggled":self.__on_checkbuttonUseProxy_toggled,
            "on_entryProxyAddress_changed":self.__on_entryProxyAddress_changed,
            "on_entryProxyPort_changed":self.__on_entryProxyPort_changed,
            "on_checkbuttonISUse_toggled":self.__on_checkbuttonISUse_toggled,
            "on_checkbuttonRSUse_toggled":self.__on_checkbuttonRSUse_toggled,
#            "on_entryISname_changed":self.__on_entryISname_changed,
#            "on_entryRSname_changed":self.__on_entryRSname_changed,
#            "on_entryISPass_changed":self.__on_entryISPass_changed,
#            "on_entryRSPass_changed":self.__on_entryRSPass_changed,
#            "on_entryProxyUserName_changed":self.__on_entryProxyUserName_changed,
#            "on_entryProxyPassword_changed":self.__on_entryProxyPassword_changed,
            "on_comboboxProxyAuthentication_changed":self.__on_comboboxProxyAuthentication_changed
        }
        self.__wTree.signal_autoconnect(dic)

        self.__checkInstalledComponents()
        self.__loadSettings()

        self.__window.show()
        #hide not yet implemented elements
        self.__wTree.get_widget("buttonInstallOptipng").hide()
        self.__wTree.get_widget("entryISName").hide()
        self.__wTree.get_widget("entryISPass").hide()
        self.__wTree.get_widget("checkbuttonISUse").hide()
        self.__wTree.get_widget("label17").hide()


    def __destroy(self, widget=None):
        set = self.__set
        set.setISCrednetials(self.__wTree.get_widget("entryISName").get_text(), self.__wTree.get_widget("entryISPass").get_text())
        set.setRSCrednetials(self.__wTree.get_widget("entryRSName").get_text(), self.__wTree.get_widget("entryRSPass").get_text())
        set.setProxyCrednetials(self.__wTree.get_widget("entryProxyUserName").get_text(), self.__wTree.get_widget("entryProxyPassword").get_text())
        set.saveSettingsToDB()
        

    def __checkInstalledComponents(self):
        from os import system
        self.__optipngInstalled = system("command -v optipng") == 0


    def __loadSettings(self):
        self.__comboboxFileType.set_active(0)
        if not self.__optipngInstalled:
            self.__optipngBox.show()
        #self.__populateUploaders()
        self.__setComponentsToValues(self.__set.getSettings())

    def __setComponentsToValues(self,settings):
        """set widget to values from given settings"""
        self.__wTree.get_widget("checkbuttonOneInstance").set_active(settings["allowOneInstanceOnly"])
        self.__wTree.get_widget("spinbuttonHeight").set_value(settings["maximumHeight"])
        self.__wTree.get_widget("spinbuttonThumb").set_value(settings["thumbnailSize"])
        self.__wTree.get_widget("checkbuttonNotification").set_active(settings["showNotifications"])
        self.__wTree.get_widget("checkbuttonIcon").set_active(settings["showIcon"])
        self.__wTree.get_widget("checkbuttonBlink").set_active(settings["blinkIcon"])

        self.__wTree.get_widget("spinbuttonNumberOfConnections").set_value(settings["maxConnections"])
        self.__wTree.get_widget("checkbuttonUseProxy").set_active(False) # needed for Reset calls only, may be later overwritten by setCredentials call
        self.__wTree.get_widget("entryProxyAddress").set_text(settings["proxyName"])
        self.__wTree.get_widget("entryProxyPort").set_text(str(settings["proxyPort"]))
        self.__setCredentials("useISPassword","checkbuttonISUse",self.__set.getISCredentials,"entryISName","entryISPass")
        self.__setCredentials("useRSPassword","checkbuttonRSUse",self.__set.getRSCredentials,"entryRSName","entryRSPass")
        self.__setCredentials("useProxy","checkbuttonUseProxy",self.__set.getProxyCredentials,"entryProxyUserName","entryProxyPassword",False)
        self.__wTree.get_widget("comboboxProxyAuthentication").set_active(settings["proxyAuthenticationType"])

        clipboardTypeIndeks = self.convertClipoardTypeToIndeks(settings["clipboardType"])
        self.__wTree.get_widget("comboboxClipboard").set_active(clipboardTypeIndeks)

        fileTypeIndeks = self.convertFileTypeToIndex(settings["fileType"])
        self.__wTree.get_widget("comboboxFileType").set_active(fileTypeIndeks)
        self.__wTree.get_widget("hscaleCompression").set_value(settings["fileSize"])

    def convertClipoardTypeToIndeks(self,type):
        clipboardTypeIndeks = 1
        if type == "PRIMARY":
            clipboardTypeIndeks = 0
        return clipboardTypeIndeks

    def convertClipoardIndexToType(self,index):
        if index == 0:
            return "PRIMARY"
        else:
            return "CLIPBOARD"

    def convertFileTypeToIndex(self,type):
        fileTypeIndeks = 0
        if type == "jpg":
            fileTypeIndeks = 1
        elif type == "tiff":
            fileTypeIndeks = 2
        elif type == "bmp":
            fileTypeIndeks = 3
        return fileTypeIndeks

    def convertFileIndexToType(self,index):
#        if index == 0:
#            return "png"
        if index == 1:
            return "jpg"
        if index == 2:
            return "tiff"
        if index == 3:
            return "bmp"
        return "png" #png is default return it if no index is selected(fe:-1)


    def __populateUploaders(self):
        """inserts entries for each file type that have assignet uploader"""
        vbox = self.__wTree.get_widget("vboxUploaders")
        #jpg and jpeg should be handled as the same
        uploadersDic = {"jpg":["imageshack", "rapidshare"], "bmp":["imageshack", "rapidshare"], "gif":["imageshack", "rapidshare"], "png":["imageshack", "rapidshare"], "tif":["imageshack", "rapidshare"], "tiff":["imageshack", "rapidshare"], "*":["rapidshare"]}
        #copy and remove default uploaders from list,to add them at the end
        defaultUploaders = uploadersDic["*"]
        del uploadersDic["*"]

        #sort uploaders by file type
        fileTypes = uploadersDic.keys()
        fileTypes.sort()

        #put non default uploaders to vbox
        for fileType in fileTypes:
            hbox = self.__createUploderHBox(fileType,uploadersDic[fileType])
            vbox.pack_start(hbox, False, False, 0)

        #put default uploader to vbox
        hbox = self.__createUploderHBox(_("Default"),defaultUploaders)
        vbox.pack_start(hbox, False, False, 8)

        vbox.show_all()
        

    def __createUploderHBox(self, fileType, uploaders):
        """returns HBox with label with file type, and combobox with given uploaders. 
        All given uploaders should be capable of handling given fileType"""
        hbox = gtk.HBox(True, 0)
        label = gtk.Label(fileType)
        hbox.pack_start(label, False, True, 0)
        combobox =  createCombobox(uploaders)
        hbox.pack_start(combobox, False, True, 0)
        return hbox
        
    def __setCredentials(self, areCredentialsUsedSettingName, checkbuttonCredentialsUsedWidgetName, getCredentialsFunction, nameWidgetName, passWidgetName,uncheckCheckbuttonIfCredentialsAreEmpty=True):
        """areCredentialsUsedSettingName - setting name with value indicating if this credentials are used
        checkbuttonCredentialsUsedWidgetName - name of checkbutton that should show if credentials are used
        getCredentialsFunction - function that will return credentials
        nameWidgetName - name of entry widget that should be populated with credential name
        passWidgetName - name of entry widget that should be populated with credential password
        uncheckCheckbuttonIfCredentialsAreEmpty - if True or not given checkbuttonCredentialsUsedWidget will be unchecked if credentials are not used or empty
        """
        credentialsExists = False
        if self.__settings[areCredentialsUsedSettingName]:
            self.__wTree.get_widget(checkbuttonCredentialsUsedWidgetName).set_active(True)
            credentials = getCredentialsFunction()
            if credentials:
                user,password = credentials
                credentialsExists = True
                self.__wTree.get_widget(nameWidgetName).set_text(user)
                self.__wTree.get_widget(passWidgetName).set_text(password)
                del user
                del password
        if not credentialsExists:
            self.__wTree.get_widget(nameWidgetName).set_text("")
            self.__wTree.get_widget(passWidgetName).set_text("")
            if uncheckCheckbuttonIfCredentialsAreEmpty:
                self.__wTree.get_widget(checkbuttonCredentialsUsedWidgetName).set_active(False)

    def __buttonOKClicked(self, widget):
        self.__window.destroy()


    def __buttonResetClicked(self, widget):
        self.__setComponentsToValues(self.__set.getDefaultSettings())
        #self.__optipngBox.show()
        
    def __scaleCompressionValueChanged(self, widget):
        #print "scaleCompressionValueChanged"
        val = widget.get_value()
        if (not self.__optipngInstalled) and val == 0 and self.__comboboxFileType.get_active() == 0:
            self.__hscaleCompression.set_value(1)
        

    def __comboboxFileTypeChanged(self, widget):
        #if change from png to jpg change hscaleValue, change labels
        index = self.__comboboxFileType.get_active()
        if  index != 0: # not png
            self.__optipngBox.hide()
        if index != 1: # not jpg
            self.__labelScaleMax.set_text("shorter\ntime")
            self.__hscaleCompression.set_value(0)
            if not self.__optipngInstalled and self.__comboboxFileType.get_active() == 0: # png
                self.__optipngBox.show()
                self.__hscaleCompression.set_value(1)
        else:
            self.__labelScaleMax.set_text("better\nquality")
            self.__hscaleCompression.set_value(75)

        value = self.convertFileIndexToType(index)
        self.__settings["fileType"] = value
        self.__settings["fileSize"] = self.__hscaleCompression.get_value()

        
    def __buttonInstallOptipng(self, widget):
        print "install optipng"
        #if system("command -v apt-get") == 0:
        self.__optipngBox.hide()

    def __buttonGetCurrentHeightClicked(self, widget):
        if self.__parent:
            parentSize = self.__parent.get_size()
            height = parentSize[1]
            spingButtonHeight = self.__wTree.get_widget("spinbuttonHeight")
            spingButtonHeight.set_value(height)

    def __spinbuttonNumberOfConnections_value_changed(self, widget):
        self.__settings["maxConnections"] = widget.get_value()

    def __comboboxClipboard_changed(self, widget):
        index = widget.get_active()
        value = self.convertClipoardIndexToType(index)
        self.__settings["clipboardType"] = value

    def __checkbuttonOneInstance_toggled(self, widget):
        self.__settings["allowOneInstanceOnly"] = widget.get_active()

    def __spinbuttonHeight_value_changed(self, widget):
        self.__settings["maximumHeight"] = widget.get_value()

    def __spinbuttonThumb_value_changed(self, widget):
        self.__settings["thumbnailSize"] = widget.get_value()

    def __checkbuttonNotification_toggled(self, widget):
        self.__settings["showNotifications"] = widget.get_active()

    def __checkbuttonIcon_toggled(self, widget):
        self.__settings["showIcon"] = widget.get_active()

    def __checkbuttonBlink_toggled(self, widget):
        self.__settings["blinkIcon"] = widget.get_active()

    def __on_checkbuttonUseProxy_toggled(self, widget):
        self.__settings["useProxy"] = widget.get_active()

    def __on_entryProxyAddress_changed(self, widget):
        self.__settings["proxyName"] = widget.get_text()

    def __on_entryProxyPort_changed(self, widget):
        self.__settings["proxyPort"] = int(widget.get_text())

#    def __on_entryProxyUserName_changed(self, widget):
#        self.__settings["proxyUser"] = widget.get_text()
#
#    def __on_entryProxyPassword_changed(self, widget):
#        self.__settings["proxyPassword"] = widget.get_text()

    def __on_comboboxProxyAuthentication_changed(self, widget):
        self.__settings["proxyAuthenticationType"] = widget.get_active()

    def __on_checkbuttonISUse_toggled(self, widget):
        self.__settings["useISPassword"] = widget.get_active()

    def __on_checkbuttonRSUse_toggled(self, widget):
        self.__settings["useRSPassword"] = widget.get_active()

if __name__ == "__main__":
    settings = Settings_GTK()
    gtk.main()
