#!/usr/bin/env python

# -*- coding: utf-8 -*-

#    Copyright (C) <2009>  <Sebastian Kacprzak> <naicik |at| gmail |dot| com>
#    Partial copyright <Balazs Nagy> <nxbalazs |at| gmail |dot| com> (few lines from his drop2imageshack plasma)

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import gettext
gettext.bindtextdomain('sendToImageshack', '.')
gettext.textdomain('sendToImageshack')
_ = gettext.gettext


import pycurl
from threading import Thread
#import os
#from helpers.gnomeHelper import getMimeTypes
from StringIO import StringIO
from threading import BoundedSemaphore
from xml.dom import minidom
from libs.uploadHelper import createLinks
from libs.cliHelper import printLink, printProgressToCLI
#from DB import DB
#db=DB()
from plugins.libs.pycurlHelper import getPycurlProxyAuthenticationType
from Settings import Settings
set = Settings()



acceptImageExtensions = ["jpg", "jpeg", "bmp", "gif", "png", "tif", "tiff"]
hosting_name="Imageshack"
def getAcceptedImageTypes():
    return acceptImageExtensions

def correctExtension(file):
    """checks if given string have a allowed extension"""
    #TODO: switch to checking mime types
    return file.split('.')[-1].lower() in acceptImageExtensions

def getName():
    return hosting_name

class SendToImageshack():

    def __init__(self, fileNumber, progressCallbackFunction):
        self.fileNumber = fileNumber
        self.progressCallbackFunction = progressCallbackFunction

    def parseXML(self, xml):
        """parse given xml
        returns dictionary with keys: IM, Forum, Alt Forum, HTML, Direct, Forum Thumb, Alt Forum Thumb, HTML Thumb, Twitter Link"""
        xmldoc = minidom.parse(StringIO(xml))

        imageLink = xmldoc.getElementsByTagName("image_link")[0].firstChild.data #xmldoc.childNodes[0].childNodes[1].firstChild.data
        thumbLink = xmldoc.getElementsByTagName("thumb_link")[0].firstChild.data #print xmldoc.childNodes[0].childNodes[3].firstChild.data
        adLink = xmldoc.getElementsByTagName("ad_link")[0].firstChild.data #print xmldoc.childNodes[0].childNodes[5].firstChild.data
        thumbExists = xmldoc.getElementsByTagName("thumb_exists")[0].firstChild.data #print xmldoc.childNodes[0].childNodes[7].firstChild.data
        if thumbExists == "no":
            thumbLink = imageLink # thumb was not generated, use image as a thumb
        links = createLinks(adLink, None, imageLink,thumbLink)
        return links, [imageLink, thumbLink, adLink]

    def progress(self, download_t, download_d, upload_t, upload_d):
        """passes self.fileNumber and arguments to self.progressCallbackFunction"""
        self.progressCallbackFunction(self.fileNumber, download_t, download_d, upload_t, upload_d)

    def upload(self, file, semaphore, callbackFunction):
        """can throw errors"""
        if not correctExtension(file): # usually this check is not needed, but it can help later in other script
            raise Exception(_("Invalid file extension\nYou can upload only ") + str(acceptImageExtensions), _("Invalid file extension"))
        curl = pycurl.Curl()

        curl.setopt(pycurl.URL, "http://www.imageshack.us/index.php")
        curl.setopt(pycurl.HTTPHEADER, ["Except:"])
        if(set.getUseProxy()):
            curl.setopt(pycurl.PROXY,set.getProxyName())
            curl.setopt(pycurl.PROXYPORT, set.getProxyPort())
            credentials = set.getProxyCredentials()
            if credentials:
                userNameAndPassword = credentials[0] + ":" + credentials[1]
                curl.setopt(pycurl.PROXYUSERPWD,userNameAndPassword)
                del userNameAndPassword
                del credentials
            curl.setopt(pycurl.PROXYAUTH,getPycurlProxyAuthenticationType())
        curl.setopt(pycurl.POST, 1)
        curl.setopt(pycurl.HTTPPOST, [('fileupload', (pycurl.FORM_FILE, file)), ('xml', 'yes')])

        buf = StringIO()
        curl.setopt(pycurl.WRITEFUNCTION, buf.write)
        curl.setopt(curl.NOPROGRESS, 0)
        curl.setopt(curl.PROGRESSFUNCTION, self.progress)

        semaphore.acquire()
        try:
            curl.perform()
        except pycurl.error:
            semaphore.release()
            raise Exception(file)
        semaphore.release()
        links, basicLinks = self.parseXML(buf.getvalue().strip()) # sometimes there are leading witespace in stream and minidom don't like them
#        db.addImage("Imageshack",basicLinks[0], basicLinks[1], os.path.basename(file), os.path.getsize(file),file.split('.')[-1].lower(), getMimeTypes(file))
        callbackFunction(links,self.fileNumber,file)
        

def uploadFile(file, resultCallbackFunction=printLink, progressCallbackFunction=printProgressToCLI, fileNumber = 0, semaphore = BoundedSemaphore(2), username="", password=""):
    sti = SendToImageshack(fileNumber, progressCallbackFunction)
    sti.upload(file, semaphore, resultCallbackFunction)

def uploadFiles(files, resultCallbackFunction=printLink, progressCallbackFunction=printProgressToCLI, fileNumber = 0, semaphore = BoundedSemaphore(2)):
    """uploads given files in threads
    This function is non blocking, you should give a callback function as parameter if you want to know when it ended or what is the progress
    Error will be thrown if upload was unsuccessful"""
    threadList = []
    for file in files:
        sti = SendToImageshack(fileNumber, progressCallbackFunction)
        fileNumber += 1
        t = Thread(None, sti.upload, None,(file, semaphore, resultCallbackFunction))
        t.start()
        threadList.append(t)
    return threadList


if __name__ == "__main__":
    from sys import argv
    if len(argv) > 1:
        args = argv[1:] # first argument is file name, so ignore it
        threadList = uploadFiles(args)
        for thread in threadList:
            thread.join() # wait for all threads to end

