# Rapidshare Uploader v 1.1 by Adam Poit
#
# Type these commands from the command line to use this script:
#
# python rsupload.py myfile.rar free (Uploads to rapidshare as a free user)
# python rsupload.py myfile.rar col Username Password (Uploads to a rapidshare collectors account)
# python rsupload.py myfile.rar prem Username Password (Uploads to a premium account)
#
# To upload multiple files, separate them with a # (number sign). Like this:
#
# python rsupload.py myfile.rar#anotherfile.rar#onemore.rar prem Username Password
#
# At the end, RSUploader will save all the urls to rsurls.txt
#################################################################
# project site <http://code.google.com/p/python-rapidshare-upload/> indicates that this script is GPL v 3
# I modiffied it to use with PyShare <https://launchpad.net/pyshare>  . <Sebastian Kacprzak> <naicik |at| gmail |dot| com>

from threading import Thread
import socket
import re
#import os
#import md5 # md5 module is deprecated - switched to hashlib
from hashlib import md5
from threading import BoundedSemaphore
from libs.uploadHelper import createLinks
from libs.cliHelper import printLink, printProgressToCLI
#from helpers.gnomeHelper import getMimeTypes
#from DB import DB
#db=DB()

def getAcceptedImageTypes():
    return "*"

hosting_name="Rapidshare"

def getName():
    return hosting_name


class __Rapidshare:
    def __init__(self, filename, resultCallbackFunction, progressCallbackFunction,fileNumber=0):
        self.filename = filename
        self.bufferlen = 0
        self.resultCallbackFunction = resultCallbackFunction
        self.progressCallbackFunction = progressCallbackFunction
        self.fileNumber = fileNumber

    def upload(self,semaphore, username, password):
        semaphore.acquire()
        try:
            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            sock.connect(("rapidshare.com", 80))

            sock.send('GET /cgi-bin/rsapi.cgi?sub=nextuploadserver_v1 HTTP/1.0\r\n\r\n')
            uploadserver = re.search('\r\n\r\n(\d+)', sock.recv(1000000))
            uploadserver = uploadserver.group().lstrip()
            sock.close()

            f = open(self.filename)
            filecontent = f.read()
            size = len(filecontent)
            md5hash = md5(filecontent)
            f.close()

            #print "MD5: " + md5hash.hexdigest().upper()
            #print "Uploading to rs" + uploadserver + "l3.rapidshare.com...\n"

            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            sock.connect(("rs" + uploadserver + "l3.rapidshare.com", 80))

            boundary = "---------------------632865735RS4EVER5675865"
            contentheader = boundary + "\r\nContent-Disposition: form-data; name=\"rsapi_v1\"\r\n\r\n1\r\n"

            if username:
#            if type == "prem":
                contentheader += boundary + "\r\nContent-Disposition: form-data; name=\"login\"\r\n\r\n" + username + "\r\n"
                contentheader += boundary + "\r\nContent-Disposition: form-data; name=\"password\"\r\n\r\n" + password + "\r\n"

#            if type == "col":
#                contentheader += boundary + "\r\nContent-Disposition: form-data; name=\"freeaccountid\"\r\n\r\n" + username + "\r\n"
#                contentheader += boundary + "\r\nContent-Disposition: form-data; name=\"password\"\r\n\r\n" + password + "\r\n"

            contentheader += boundary + "\r\nContent-Disposition: form-data; name=\"filecontent\"; filename=\"" + self.filename + "\"\r\n\r\n"
            contenttail = "\r\n" + boundary + "--\r\n"
            contentlength = len(contentheader) + size + len(contenttail)

            header = "POST /cgi-bin/upload.cgi HTTP/1.0\r\nContent-Type: multipart/form-data; boundary=" + boundary + "\r\nContent-Length: " + str(contentlength) + "\r\n\r\n" + contentheader

            sock.send(header)

            f = open(self.filename)

            while True:
                chunk = f.read(64000)
                if not chunk:
                    break
                sock.send(chunk)
                self.bufferlen += len(chunk)
                #pere = float(self.bufferlen) / float(size) * 100
                #rnd = round(pere, 1)
                self.progressCallbackFunction(self.fileNumber, upload_t = size,upload_d = self.bufferlen)
                #print "sending"
                #print "Sent " + str(self.bufferlen) + " of " + str(size) + " bytes / " + str(rnd) + "%"

            sock.send(contenttail)

            result = sock.recv(1000000)
            f.close()
            sock.close()

            #print "\nChecking MD5..."

            newhash = re.search('File1.4=(\w+)', result)
            #print result
            oldhash = "File1.4=" + md5hash.hexdigest().upper()
            if not newhash:
                raise Exception("Upload unsuccessful")
            if newhash.group() == oldhash:
                upload = re.search('File1.1=(\S+)', result)
                link = upload.group()[8:]#Cut'File1.1='
                links = createLinks(link, self.filename)
                self.resultCallbackFunction(links, self.fileNumber, self.filename)
#                db.addFile("Rapidshare",link, 0, os.path.basename(self.filename), os.path.getsize(self.filename), self.filename.split('.')[-1].lower(), getMimeTypes(self.filename))
                #print "File uploaded successfully: " + link + "\n\n"
                #f = open("rsurls.txt", "a")
                #f.write(upload.group() + "\n")
                #f.close()
            else:
                #print "Upload Failure\n"
                raise Exception("Upload unsuccessful")
        finally:
            semaphore.release()

def printProgressToCLI(fileNumber, upload_t, upload_d):
    pass

def printLink(link, fileNumber,filename):
    print link


def uploadFile(file, resultCallbackFunction=printLink, progressCallbackFunction=printProgressToCLI, fileNumber = 0, semaphore = BoundedSemaphore(2), username="", password=""):
    rs = __Rapidshare(file, resultCallbackFunction, progressCallbackFunction, fileNumber)
    from Settings import Settings
    set = Settings()
    if set.getUseRSPassword():
        username,password = set.getRSCredentials()
    rs.upload(semaphore, username, password)
    del username
    del password


def uploadFiles(files, resultCallbackFunction=printLink, progressCallbackFunction=printProgressToCLI, fileNumber = 0, semaphore = BoundedSemaphore(2), username="", password=""):
    for file in files:
        rs = __Rapidshare(file, resultCallbackFunction, progressCallbackFunction, fileNumber)
        fileNumber += 1
        t = Thread(None, rs.upload, None,(semaphore, username, password))
        t.start()
           
        

if __name__ == "__main__":
    import sys
    filename = sys.argv[1]
    type = sys.argv[2]
    username = password = ""
    if type != "free":
        username = sys.argv[3]
        password = sys.argv[4]

    files = filename.split('#')

    for file in files:
        rs = __Rapidshare(file,printLink,printProgressToCLI,0)
        rs.upload(BoundedSemaphore(), username, password)
